from contextlib import AbstractAsyncContextManager, AbstractContextManager
from http import HTTPStatus
from typing import Literal

import httpx

from helpdesk_client.utils import raise_for_status
from helpdesk_client.v3.dto import UploadFileDTO
from helpdesk_client.v3.schemas.body import (
    MainNoteCreateSchema,
    MainRequestCreateUpdateSchema,
    NoteCreateSchema,
    RequestCreateSchema,
    RequestUpdateSchema,
    TemplateSchema,
)
from helpdesk_client.v3.schemas.query_params import (
    CategoryFilterParams,
    HelpdeskFilter,
    RequestFilterPagePaginationParams,
    RequestFilterParams,
    SubcategoryFilterParams,
    TemplateFilterParams,
    UrgencyFilterParams,
)
from helpdesk_client.v3.schemas.response import (
    CategoryPaginationResponseSchema,
    MainNoteSchema,
    MainRequestAttachmentSchema,
    MainResolutionSchema,
    NoteSchema,
    RequestAttachmentSchema,
    RequestPaginationResponseSchema,
    RequestSchema,
    ResolutionSchema,
    ServiceCategoryPaginationResponseSchema,
    SubcategoryPaginationResponseSchema,
    TemplatePaginationResponseSchema,
    UrgencyPaginationResponseSchema,
)

from .schemas import MainRequestSchema, RequestListSchema
from .urls import HelpdeskUrls


class HelpdeskClient:
    def __init__(
        self,
        http_client: httpx.AsyncClient,
        urls: HelpdeskUrls | None = None,
    ) -> None:
        self._http_client = http_client
        self._urls = urls or HelpdeskUrls()

    async def get_request(self, ident: int) -> RequestSchema | None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.request_by_id(ident)
        response = await self._http_client.get(url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        schema = MainRequestSchema.model_validate(response.json())
        return schema.request

    async def get_requests(
        self,
        filter_: RequestFilterParams,
    ) -> RequestListSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(self._urls.requests, params=params)
        raise_for_status(response)
        return RequestListSchema.model_validate(response.json())

    async def get_requests_page_paginated(
        self,
        filter_: RequestFilterPagePaginationParams,
    ) -> RequestPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(self._urls.requests, params=params)
        raise_for_status(response)
        return RequestPaginationResponseSchema.model_validate(response.json())

    async def create_request(
        self,
        schema: RequestCreateSchema,
    ) -> RequestSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        main_schema = MainRequestCreateUpdateSchema(request=schema)
        body = {
            "input_data": main_schema.model_dump_json(
                by_alias=True,
                exclude_unset=True,
            ),
        }
        response = await self._http_client.post(self._urls.requests, data=body)
        raise_for_status(response)
        response_schema = MainRequestSchema.model_validate(response.json())
        return response_schema.request

    async def update_request(
        self,
        ident: int,
        schema: RequestUpdateSchema,
    ) -> RequestSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`, `ValueError`"""

        if schema.is_empty:
            msg = "Got schema with empty values"
            raise ValueError(msg)

        url = self._urls.request_by_id(ident)
        main_schema = MainRequestCreateUpdateSchema(request=schema)
        body = {
            "input_data": main_schema.model_dump_json(
                by_alias=True,
                exclude_unset=True,
            ),
        }
        response = await self._http_client.put(url, data=body)
        raise_for_status(response)
        response_schema = MainRequestSchema.model_validate(response.json())
        return response_schema.request

    async def cancel_request(
        self,
        request_id: int,
    ) -> None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.cancel_request(request_id)
        response = await self._http_client.put(url)
        raise_for_status(response)

    async def attach_file_to_request(
        self,
        request_id: int,
        dto: UploadFileDTO,
        file_field: Literal["file", "input_file"] = "input_file",
    ) -> RequestAttachmentSchema:
        """
        Прикрепляет файл к заявке.

        :param request_id: Идентификатор заявки
        :param dto: `UploadFileDTO`
        :param file_field: Наименование поля файла. В версии ServiceDesk 14.8 вместо `file` ожидается `input_file`
        raises: `HelpdeskClientError`, `httpx.HTTPError`
        """

        url = self._urls.upload_file(request_id)
        files = {file_field: (dto.filename, dto.file, dto.content_type)}
        response = await self._http_client.put(url, files=files)
        raise_for_status(response)
        response_schema = MainRequestAttachmentSchema.model_validate(response.json())
        return response_schema.attachment

    async def get_categories(
        self,
        filter_: CategoryFilterParams,
    ) -> CategoryPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(self._urls.categories, params=params)
        raise_for_status(response)
        return CategoryPaginationResponseSchema.model_validate(response.json())

    async def get_service_categories(
        self,
        filter_: CategoryFilterParams,
    ) -> ServiceCategoryPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(
            self._urls.service_categories,
            params=params,
        )
        raise_for_status(response)
        return ServiceCategoryPaginationResponseSchema.model_validate(response.json())

    async def get_subcategories(
        self,
        filter_: SubcategoryFilterParams,
    ) -> SubcategoryPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(self._urls.subcategories, params=params)
        raise_for_status(response)
        return SubcategoryPaginationResponseSchema.model_validate(response.json())

    async def get_templates(
        self,
        filter_: TemplateFilterParams,
    ) -> TemplatePaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.request_template
        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(url, params=params)
        raise_for_status(response)
        return TemplatePaginationResponseSchema.model_validate(response.json())

    async def get_template(
        self,
        ident: int,
    ) -> TemplateSchema | None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.template_by_id(ident)
        response = await self._http_client.get(url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        return TemplateSchema.model_validate(response.json())

    async def get_urgencies(
        self,
        filter_: UrgencyFilterParams,
    ) -> UrgencyPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = await self._http_client.get(self._urls.urgencies, params=params)
        raise_for_status(response)
        return UrgencyPaginationResponseSchema.model_validate(response.json())

    async def add_note(
        self,
        request_id: int,
        schema: NoteCreateSchema,
    ) -> NoteSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.create_note(request_id)
        main_schema = MainNoteCreateSchema(note=schema)
        body = {
            "input_data": main_schema.model_dump_json(
                by_alias=True,
                exclude_unset=True,
            ),
        }
        response = await self._http_client.post(url, data=body)
        raise_for_status(response)
        return MainNoteSchema.model_validate(response.json()).note

    async def attach_file_to_note(
        self,
        request_id: int,
        note_id: int,
        dto: UploadFileDTO,
        file_field: Literal["file", "input_file"] = "input_file",
    ) -> RequestAttachmentSchema:
        """
        Прикрепляет файл к комментарию/обсуждению.

        :param request_id: Идентификатор заявки
        :param note_id: Идентификатор комментария/обсуждения
        :param dto: `UploadFileDTO`
        :param file_field: Наименование поля файла. В версии ServiceDesk 14.8 вместо `file` ожидается `input_file`
        raises: `HelpdeskClientError`, `httpx.HTTPError`
        """

        url = self._urls.upload_note_file(request_id=request_id, note_id=note_id)
        files = {file_field: (dto.filename, dto.file, dto.content_type)}
        response = await self._http_client.put(url, files=files)
        raise_for_status(response)
        response_schema = MainRequestAttachmentSchema.model_validate(response.json())
        return response_schema.attachment

    async def get_resolution(
        self,
        request_id: int,
    ) -> ResolutionSchema | None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.resolutions(request_id)
        response = await self._http_client.get(url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        return MainResolutionSchema.model_validate(response.json()).resolution

    async def download(self, content_url: str) -> bytes | None:
        """
        Скачивает ресурс по указанному URL.

        :param content_url: Примеры: 1) `content_url` из `RequestAttachmentSchema`; 2) значение атрибута `src` тега `img` из `ResolutionSchema.raw_content`.
        raises: `HelpdeskClientError`, `httpx.HTTPError`
        """

        if content_url.startswith("/"):
            content_url = content_url.removeprefix("/")

        response = await self._http_client.get(content_url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        return response.content

    def stream(self, content_url: str) -> AbstractAsyncContextManager[httpx.Response]:
        """
        Стримит ресурс по указанному URL.

        :param content_url: Примеры: 1) `content_url` из `RequestAttachmentSchema`; 2) значение атрибута `src` тега `img` из `ResolutionSchema.raw_content`.
        raises: `httpx.HTTPError`
        """

        if content_url.startswith("/"):
            content_url = content_url.removeprefix("/")

        return self._http_client.stream("GET", content_url)


class SyncHelpdeskClient:
    def __init__(
        self,
        http_client: httpx.Client,
        urls: HelpdeskUrls | None = None,
    ) -> None:
        self._http_client = http_client
        self._urls = urls or HelpdeskUrls()

    def get_request(self, ident: int) -> RequestSchema | None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.request_by_id(ident)
        response = self._http_client.get(url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        schema = MainRequestSchema.model_validate(response.json())
        return schema.request

    def get_requests(
        self,
        filter_: RequestFilterParams,
    ) -> RequestListSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(self._urls.requests, params=params)
        raise_for_status(response)
        return RequestListSchema.model_validate(response.json())

    def get_requests_page_paginated(
        self,
        filter_: RequestFilterPagePaginationParams,
    ) -> RequestPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(self._urls.requests, params=params)
        raise_for_status(response)
        return RequestPaginationResponseSchema.model_validate(response.json())

    def create_request(
        self,
        schema: RequestCreateSchema,
    ) -> RequestSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        main_schema = MainRequestCreateUpdateSchema(request=schema)
        body = {
            "input_data": main_schema.model_dump_json(
                by_alias=True,
                exclude_unset=True,
            ),
        }
        response = self._http_client.post(self._urls.requests, data=body)
        raise_for_status(response)
        response_schema = MainRequestSchema.model_validate(response.json())
        return response_schema.request

    def update_request(
        self,
        ident: int,
        schema: RequestUpdateSchema,
    ) -> RequestSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`, `ValueError`"""

        if schema.is_empty:
            msg = "Got schema with empty values"
            raise ValueError(msg)

        url = self._urls.request_by_id(ident)
        main_schema = MainRequestCreateUpdateSchema(request=schema)
        body = {
            "input_data": main_schema.model_dump_json(
                by_alias=True,
                exclude_unset=True,
            ),
        }
        response = self._http_client.put(url, data=body)
        raise_for_status(response)
        response_schema = MainRequestSchema.model_validate(response.json())
        return response_schema.request

    def cancel_request(
        self,
        request_id: int,
    ) -> None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.cancel_request(request_id)
        response = self._http_client.put(url)
        raise_for_status(response)

    def attach_file_to_request(
        self,
        request_id: int,
        dto: UploadFileDTO,
        file_field: Literal["file", "input_file"] = "input_file",
    ) -> RequestAttachmentSchema:
        """
        Прикрепляет файл к заявке.

        :param request_id: Идентификатор заявки
        :param dto: `UploadFileDTO`
        :param file_field: Наименование поля файла. В версии ServiceDesk 14.8 вместо `file` ожидается `input_file`
        raises: `HelpdeskClientError`, `httpx.HTTPError`
        """

        url = self._urls.upload_file(request_id)
        files = {file_field: (dto.filename, dto.file, dto.content_type)}
        response = self._http_client.put(url, files=files)
        raise_for_status(response)
        response_schema = MainRequestAttachmentSchema.model_validate(response.json())
        return response_schema.attachment

    def get_categories(
        self,
        filter_: CategoryFilterParams,
    ) -> CategoryPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(self._urls.categories, params=params)
        raise_for_status(response)
        return CategoryPaginationResponseSchema.model_validate(response.json())

    def get_service_categories(
        self,
        filter_: CategoryFilterParams,
    ) -> ServiceCategoryPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(
            self._urls.service_categories,
            params=params,
        )
        raise_for_status(response)
        return ServiceCategoryPaginationResponseSchema.model_validate(response.json())

    def get_subcategories(
        self,
        filter_: SubcategoryFilterParams,
    ) -> SubcategoryPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(self._urls.subcategories, params=params)
        raise_for_status(response)
        return SubcategoryPaginationResponseSchema.model_validate(response.json())

    def get_templates(
        self,
        filter_: TemplateFilterParams,
    ) -> TemplatePaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.request_template
        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(url, params=params)
        raise_for_status(response)
        return TemplatePaginationResponseSchema.model_validate(response.json())

    def get_template(
        self,
        ident: int,
    ) -> TemplateSchema | None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.template_by_id(ident)
        response = self._http_client.get(url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        return TemplateSchema.model_validate(response.json())

    def get_urgencies(
        self,
        filter_: UrgencyFilterParams,
    ) -> UrgencyPaginationResponseSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        schema = HelpdeskFilter(list_info=filter_)
        params = {
            "input_data": schema.model_dump_json(by_alias=True, exclude_unset=True),
        }
        response = self._http_client.get(self._urls.urgencies, params=params)
        raise_for_status(response)
        return UrgencyPaginationResponseSchema.model_validate(response.json())

    def add_note(
        self,
        request_id: int,
        schema: NoteCreateSchema,
    ) -> NoteSchema:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.create_note(request_id)
        main_schema = MainNoteCreateSchema(note=schema)
        body = {
            "input_data": main_schema.model_dump_json(
                by_alias=True,
                exclude_unset=True,
            ),
        }
        response = self._http_client.post(url, data=body)
        raise_for_status(response)
        return MainNoteSchema.model_validate(response.json()).note

    def attach_file_to_note(
        self,
        request_id: int,
        note_id: int,
        dto: UploadFileDTO,
        file_field: Literal["file", "input_file"] = "input_file",
    ) -> RequestAttachmentSchema:
        """
        Прикрепляет файл к комментарию/обсуждению.

        :param request_id: Идентификатор заявки
        :param note_id: Идентификатор комментария/обсуждения
        :param dto: `UploadFileDTO`
        :param file_field: Наименование поля файла. В версии ServiceDesk 14.8 вместо `file` ожидается `input_file`
        raises: `HelpdeskClientError`, `httpx.HTTPError`
        """

        url = self._urls.upload_note_file(request_id=request_id, note_id=note_id)
        files = {file_field: (dto.filename, dto.file, dto.content_type)}
        response = self._http_client.put(url, files=files)
        raise_for_status(response)
        response_schema = MainRequestAttachmentSchema.model_validate(response.json())
        return response_schema.attachment

    def get_resolution(
        self,
        request_id: int,
    ) -> ResolutionSchema | None:
        """raises: `HelpdeskClientError`, `httpx.HTTPError`"""

        url = self._urls.resolutions(request_id)
        response = self._http_client.get(url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        return MainResolutionSchema.model_validate(response.json()).resolution

    def download(self, content_url: str) -> bytes | None:
        """
        Скачивает ресурс по указанному URL.

        :param content_url: Примеры: 1) `content_url` из `RequestAttachmentSchema`; 2) значение атрибута `src` тега `img` из `ResolutionSchema.raw_content`.
        raises: `HelpdeskClientError`, `httpx.HTTPError`
        """

        if content_url.startswith("/"):
            content_url = content_url.removeprefix("/")

        response = self._http_client.get(content_url)
        if response.status_code == HTTPStatus.NOT_FOUND:
            return None

        raise_for_status(response)
        return response.content

    def stream(self, content_url: str) -> AbstractContextManager[httpx.Response]:
        """
        Стримит ресурс по указанному URL.

        :param content_url: Примеры: 1) `content_url` из `RequestAttachmentSchema`; 2) значение атрибута `src` тега `img` из `ResolutionSchema.raw_content`.
        raises: `httpx.HTTPError`
        """

        if content_url.startswith("/"):
            content_url = content_url.removeprefix("/")

        return self._http_client.stream("GET", content_url)



import re

import httpx
from pydantic import BaseModel

from .exceptions import HelpdeskClientError


def raise_for_status(response: httpx.Response) -> None:
    if response.is_success:
        return

    raise HelpdeskClientError(
        status_code=response.status_code,
        response_data=response.content,
    )


def json_dump_model(value: BaseModel | None) -> str | None:
    if value is None:
        return None

    return value.model_dump_json(exclude_unset=True, by_alias=True)


def remove_html_tags(value: str) -> str:
    return re.sub(r"<[^<]+?>", "", value)



from contextlib import suppress
from functools import cached_property
from typing import Any

import orjson
from orjson import JSONDecodeError


class HelpdeskClientError(Exception):
    status_code: int
    response_data: str

    def __init__(self, status_code: int, response_data: bytes) -> None:
        self.status_code = status_code
        self.response_data = response_data.decode()

    def __str__(self) -> str:
        return f"HTTP CODE: {self.status_code}, RESPONSE DATA: {self.response_data}"

    @cached_property
    def json_data(self) -> dict[str, Any] | None:
        if not self.response_data:
            return None

        with suppress(JSONDecodeError):
            return orjson.loads(self.response_data) # type: ignore[no-any-return]

        return None



from typing import Any

import pydantic

from helpdesk_client.types_ import BaseSchema


class IdentSchema(BaseSchema):
    id: int


class TemplateSchema(BaseSchema):
    id: int
    is_service_template: bool | None = None
    service_category: IdentSchema | None = None
    """Subcategory"""


class ShortRequesterSchema(BaseSchema):
    id: int | None = None
    name: str | None = None
    email: str | None = pydantic.Field(None, alias="email_id")
    phone: str | None = None


class RequestCreateSchema(BaseSchema):
    subject: str
    description: str
    requester: ShortRequesterSchema
    template: TemplateSchema | None = None
    urgency: IdentSchema
    mode: IdentSchema | None = None
    udf_fields: dict[str, Any] | None = None


class RequestUpdateSchema(BaseSchema):
    subject: str | None = None
    description: str | None = None
    urgency: IdentSchema | None = None

    @property
    def is_empty(self) -> bool:
        return all(value is None for _, value in self)


class MainRequestCreateUpdateSchema(BaseSchema):
    request: RequestCreateSchema | RequestUpdateSchema


class NoteCreateSchema(BaseSchema):
    description: str
    show_to_requester: bool
    mark_first_response: bool
    add_to_linked_requests: bool


class MainNoteCreateSchema(BaseSchema):
    note: NoteCreateSchema



from .client import HelpdeskClient, SyncHelpdeskClient
from .dto import UploadFileDTO
from .schemas import (
    CategoryFilterParams,
    CategoryPaginationResponseSchema,
    CategorySchema,
    CategorySearchFields,
    DateTimeSchema,
    FileSizeSchema,
    HasNameSchema,
    HelpdeskFilter,
    IdentSchema,
    MainNoteCreateSchema,
    MainNoteSchema,
    MainRequestAttachmentSchema,
    MainRequestCreateUpdateSchema,
    MainRequestSchema,
    MainResolutionSchema,
    NoteCreateSchema,
    NoteSchema,
    OrderingParams,
    PaginationBaseResponse,
    PaginationInfo,
    PaginationResponseSchema,
    RequestAttachmentSchema,
    RequestCreateSchema,
    RequesterSchema,
    RequestFilterParams,
    RequestListSchema,
    RequestSchema,
    RequestSearchFields,
    RequestUpdateSchema,
    ResolutionSchema,
    ShortCategorySchema,
    ShortRequesterSchema,
    ShortSubcategorySchema,
    ShortUrgencySchema,
    SubcategoryFilterParams,
    SubcategoryPaginationResponseSchema,
    SubcategorySchema,
    SubcategorySearchFields,
    TemplateFilterParams,
    TemplateSchema,
    TemplateSearchFields,
    UrgencyFilterParams,
    UrgencyPaginationResponseSchema,
    UrgencySchema,
    UrgencySearchFields,
)
from .urls import HelpdeskUrls

__all__ = [
    "CategoryFilterParams",
    "CategoryPaginationResponseSchema",
    "CategorySchema",
    "CategorySearchFields",
    "DateTimeSchema",
    "FileSizeSchema",
    "HasNameSchema",
    "HelpdeskClient",
    "HelpdeskFilter",
    "HelpdeskUrls",
    "IdentSchema",
    "MainNoteCreateSchema",
    "MainNoteSchema",
    "MainRequestAttachmentSchema",
    "MainRequestCreateUpdateSchema",
    "MainRequestSchema",
    "MainResolutionSchema",
    "NoteCreateSchema",
    "NoteSchema",
    "OrderingParams",
    "PaginationBaseResponse",
    "PaginationInfo",
    "PaginationResponseSchema",
    "RequestAttachmentSchema",
    "RequestCreateSchema",
    "RequesterSchema",
    "RequestFilterParams",
    "RequestListSchema",
    "RequestSchema",
    "RequestSearchFields",
    "RequestUpdateSchema",
    "ResolutionSchema",
    "ShortCategorySchema",
    "ShortRequesterSchema",
    "ShortSubcategorySchema",
    "ShortUrgencySchema",
    "SubcategoryFilterParams",
    "SubcategoryPaginationResponseSchema",
    "SubcategorySchema",
    "SubcategorySearchFields",
    "SyncHelpdeskClient",
    "TemplateFilterParams",
    "TemplateSchema",
    "TemplateSearchFields",
    "UploadFileDTO",
    "UrgencyFilterParams",
    "UrgencyPaginationResponseSchema",
    "UrgencySchema",
    "UrgencySearchFields",
]



from dataclasses import dataclass


@dataclass(frozen=True, slots=True)
class HelpdeskUrls:
    v3 = "api/v3"
    requests = f"{v3}/requests"
    request_template = f"{v3}/request_templates"
    categories = f"{v3}/categories"
    service_categories = f"{v3}/service_categories"
    subcategories = f"{v3}/subcategories"
    urgencies = f"{v3}/urgencies"

    @classmethod
    def template_by_id(cls, ident: int) -> str:
        return f"{cls.request_template}/{ident}"

    @classmethod
    def request_by_id(cls, request_id: int) -> str:
        return f"{cls.requests}/{request_id}"

    @classmethod
    def cancel_request(cls, request_id: int) -> str:
        return f"{cls.requests}/{request_id}/cancel"

    @classmethod
    def upload_file(cls, request_id: int) -> str:
        return f"{cls.requests}/{request_id}/upload"

    @classmethod
    def create_note(cls, request_id: int) -> str:
        return f"{cls.requests}/{request_id}/notes"

    @classmethod
    def upload_note_file(cls, request_id: int, note_id: int) -> str:
        return f"{cls.requests}/{request_id}/notes/{note_id}/upload"

    @classmethod
    def resolutions(cls, request_id: int) -> str:
        return f"{cls.requests}/{request_id}/resolutions"



from collections.abc import Sequence
from datetime import datetime
from typing import Annotated

import pydantic

from helpdesk_client.types_ import BaseSchema
from helpdesk_client.utils import remove_html_tags

from .pagination import PagePaginationInfo, PaginationInfo


class PaginationResponseSchema(PaginationInfo, BaseSchema):
    has_next: bool = pydantic.Field(alias="has_more_rows")
    total_count: int | None = None
    page_: int | None = pydantic.Field(None, alias="page")
    page_number: int | None = None

    @property
    def total_count_(self) -> int:
        if self.total_count is None:
            raise ValueError

        return self.total_count

    @property
    def page(self) -> int:
        if self.page_ is None and self.page_number is None:
            raise ValueError

        return self.page_ or self.page_number  # type: ignore[return-value]


class PaginationBaseResponse(BaseSchema):
    list_info: PaginationResponseSchema


class PagePaginationResponseSchema(PagePaginationInfo, BaseSchema):
    has_next: bool = pydantic.Field(alias="has_more_rows")
    total_count: int | None = None


class PagePaginationBaseResponse(BaseSchema):
    list_info: PagePaginationResponseSchema


class HasNameSchema(BaseSchema):
    name: str


class RequesterSchema(BaseSchema):
    id: int
    email: str | None = pydantic.Field(None, alias="email_id")
    phone: str | None = None
    name: str | None = None


class DateTimeSchema(BaseSchema):
    display_value: str
    """Not in UTC"""

    value: datetime
    """In UTC"""


class RequestSchema(BaseSchema):
    id: int
    subject: str | None = None
    description: str | None = None
    created_time: DateTimeSchema
    due_by_time: DateTimeSchema | None = None
    completed_time: DateTimeSchema | None = None
    group: HasNameSchema
    status: HasNameSchema
    requester: RequesterSchema
    technician: RequesterSchema | None = None
    attachments: Sequence["RequestAttachmentSchema"] = pydantic.Field(
        default_factory=list,
    )
    urgency: "ShortUrgencySchema | None" = None


class RequestListSchema(BaseSchema):
    requests: list[RequestSchema]


class MainRequestSchema(BaseSchema):
    request: RequestSchema


class CategorySchema(BaseSchema):
    id: int
    name: str
    description: str | None = None
    is_deleted: bool = pydantic.Field(alias="deleted")


class ShortCategorySchema(BaseSchema):
    id: int
    name: str


class SubcategorySchema(BaseSchema):
    id: int
    name: str
    description: str | None = None
    is_deleted: bool | None = pydantic.Field(alias="deleted")
    category: ShortCategorySchema


class ShortSubcategorySchema(BaseSchema):
    id: int
    name: str


class ShortUrgencySchema(BaseSchema):
    id: int
    name: str


class UrgencySchema(BaseSchema):
    id: int
    name: str
    description: str | None = None
    is_deleted: bool = pydantic.Field(alias="deleted")


class RequestPaginationResponseSchema(PagePaginationBaseResponse):
    requests: Sequence[RequestSchema]


class CategoryPaginationResponseSchema(PaginationBaseResponse):
    categories: Sequence[CategorySchema]


class ServiceCategoryPaginationResponseSchema(PaginationBaseResponse):
    service_categories: Sequence[CategorySchema]


class SubcategoryPaginationResponseSchema(PaginationBaseResponse):
    subcategories: Sequence[SubcategorySchema]


class UrgencyPaginationResponseSchema(PaginationBaseResponse):
    urgencies: Sequence[UrgencySchema]


class FileSizeSchema(BaseSchema):
    display_value: str
    value: int
    """In bytes"""


class RequestAttachmentSchema(BaseSchema):
    id: int
    name: str
    content_url: str
    description: str | None = None
    attached_by: RequesterSchema
    content_type: str | None = None
    attached_on: DateTimeSchema
    size: FileSizeSchema


class MainRequestAttachmentSchema(BaseSchema):
    attachment: RequestAttachmentSchema


class NoteSchema(BaseSchema):
    id: int
    description: str
    added_by: RequesterSchema
    added_time: DateTimeSchema
    show_to_requester: bool


class MainNoteSchema(BaseSchema):
    note: NoteSchema


class ResolutionSchema(BaseSchema):
    content: Annotated[str, pydantic.BeforeValidator(remove_html_tags)]
    raw_content: Annotated[str, pydantic.Field(alias="content")]
    attachments: Annotated[
        list[RequestAttachmentSchema],
        pydantic.Field(alias="resolution_attachments"),
    ]
    submitted_by: RequesterSchema
    submitted_on: DateTimeSchema


class MainResolutionSchema(BaseSchema):
    resolution: ResolutionSchema


class TemplateSchema(BaseSchema):
    id: int
    name: str
    is_service_template: bool
    is_enabled: bool
    inactive: bool
    is_default_template: bool


class TemplatePaginationResponseSchema(PaginationBaseResponse):
    request_templates: Sequence[TemplateSchema]



from typing import Annotated

import pydantic
from pydantic import PlainSerializer

from helpdesk_client.enums import SortEnum
from helpdesk_client.types_ import BaseSchema
from helpdesk_client.utils import json_dump_model

from .pagination import PagePaginationInfo, PaginationInfo


class OrderingParams(BaseSchema):
    sort_field: str | None = None
    sort_order: SortEnum | None = None


class RequestSearchFields(BaseSchema):
    requester_name: str | None = pydantic.Field(None, alias="requester.name")
    requester_id: int | None = pydantic.Field(None, alias="requester.id")


class CategorySearchFields(BaseSchema):
    is_deleted: bool | None = pydantic.Field(None, alias="deleted")
    name: str | None = None


class SubcategorySearchFields(BaseSchema):
    is_deleted: bool | None = pydantic.Field(None, alias="deleted")
    name: str | None = None
    category_name: str | None = pydantic.Field(None, alias="category.name")


class UrgencySearchFields(BaseSchema):
    is_deleted: bool | None = pydantic.Field(None, alias="deleted")
    name: str | None = None


class TemplateSearchFields(BaseSchema):
    is_service_template: bool | None = None
    name: str | None = None
    service_category_id: int | None = pydantic.Field(None, alias="service_category")


JsonDumpedRequestSearchFields = Annotated[
    RequestSearchFields | None,
    PlainSerializer(lambda model: json_dump_model(model), when_used="json"),
]


class RequestFilterParams(PaginationInfo, OrderingParams, BaseSchema):
    search_fields: JsonDumpedRequestSearchFields = None


class RequestFilterPagePaginationParams(PagePaginationInfo, OrderingParams, BaseSchema):
    search_fields: JsonDumpedRequestSearchFields = None


class CategoryFilterParams(PaginationInfo, OrderingParams, BaseSchema):
    search_fields: Annotated[
        CategorySearchFields | None,
        PlainSerializer(lambda model: json_dump_model(model), when_used="json"),
    ] = None


class SubcategoryFilterParams(PaginationInfo, OrderingParams, BaseSchema):
    search_fields: Annotated[
        SubcategorySearchFields | None,
        PlainSerializer(lambda model: json_dump_model(model), when_used="json"),
    ] = None


class UrgencyFilterParams(PaginationInfo, OrderingParams, BaseSchema):
    search_fields: Annotated[
        UrgencySearchFields | None,
        PlainSerializer(lambda model: json_dump_model(model), when_used="json"),
    ] = None


class TemplateFilterParams(PaginationInfo, OrderingParams, BaseSchema):
    search_fields: Annotated[
        TemplateSearchFields | None,
        PlainSerializer(lambda model: json_dump_model(model), when_used="json"),
    ] = None


class HelpdeskFilter(BaseSchema):
    list_info: (
        RequestFilterParams
        | RequestFilterPagePaginationParams
        | CategoryFilterParams
        | SubcategoryFilterParams
        | UrgencyFilterParams
        | TemplateFilterParams
    )



import pydantic

from helpdesk_client.types_ import BaseSchema


class PaginationInfo(BaseSchema):
    limit: int = pydantic.Field(alias="row_count")
    offset: int = pydantic.Field(alias="start_index")
    can_include_count: bool = pydantic.Field(default=False, alias="get_total_count")


class PagePaginationInfo(BaseSchema):
    page: int
    page_size: int = pydantic.Field(alias="row_count")
    can_include_count: bool = pydantic.Field(default=False, alias="get_total_count")
